@@ -41,17 +41,18 @@ class Task(object):
         auto_shortflags=True,
         help=None,
         pre=None,
+        post=None,
     ):
         # Real callable
         self.body = body
-        # Must copy doc/name here because Sphinx is retarded about properties.
+        # Must copy doc/name here because Sphinx is stupid about properties.
         self.__doc__ = getattr(body, '__doc__', '')
         self.__name__ = getattr(body, '__name__', '')
         # Is this a contextualized task?
         self.contextualized = contextualized
         # Default name, alternate names, and whether it should act as the
         # default for its parent collection
-        self.name = name
+        self._name = name
         self.aliases = aliases
         self.is_default = default
         # Arg/flag/parser hints
@@ -61,8 +62,45 @@ class Task(object):
         self.help = help or {}
         # Call chain bidness
         self.pre = pre or []
+        self.post = post or []
         self.times_called = 0
 
+    @property
+    def name(self):
+        return self._name or self.__name__
+
+    def __str__(self):
+        aliases = ""
+        if self.aliases:
+            aliases = " ({0})".format(', '.join(self.aliases))
+        return "<Task {0!r}{1}>".format(self.name, aliases)
+
+    def __repr__(self):
+        return str(self)
+
+    def __eq__(self, other):
+        if self.name != other.name:
+            return False
+        # Functions do not define __eq__ but func_code objects apparently do.
+        # (If we're wrapping some other callable, they will be responsible for
+        # defining equality on their end.)
+        if self.body == other.body:
+            return True
+        else:
+            try:
+                return (
+                    six.get_function_code(self.body) ==
+                    six.get_function_code(other.body)
+                )
+            except AttributeError:
+                return False
+
+    def __hash__(self):
+        # Presumes name and body will never be changed. Hrm.
+        # Potentially cleaner to just not use Tasks as hash keys, but let's do
+        # this for now.
+        return hash(self.name) + hash(self.body)
+
     def __call__(self, *args, **kwargs):
         # Guard against calling contextualized tasks with no context.
         if self.contextualized and not isinstance(args[0], Context):
@@ -114,6 +152,10 @@ class Task(object):
 
     def arg_opts(self, name, default, taken_names):
         opts = {}
+        # Whether it's positional or not
+        opts['positional'] = name in self.positional
+        # Whether it is a value-optional flag
+        opts['optional'] = name in self.optional
         # Argument name(s) (replace w/ dashed version if underscores present,
         # and move the underscored version to be the attr_name instead.)
         if '_' in name:
@@ -135,10 +177,6 @@ class Task(object):
         # Help
         if name in self.help:
             opts['help'] = self.help[name]
-        # Whether it's positional or not
-        opts['positional'] = name in self.positional
-        # Whether it is a value-optional flag
-        opts['optional'] = name in self.optional
         return opts
 
     def get_arguments(self):
@@ -210,8 +248,8 @@ def task(*args, **kwargs):
       flags from task options; defaults to True.
     * ``help``: Dict mapping argument names to their help strings. Will be
       displayed in ``--help`` output.
-    * ``pre``: List of task names, for tasks that should get run prior to the
-      wrapped task whenever it is executed via the command line.
+    * ``pre``: List of task objects to execute prior to the
+      wrapped task whenever it is executed.
 
     If any non-keyword arguments are given, they are taken as the value of the
     ``pre`` kwarg for convenience's sake. (It is an error to give both
@@ -220,7 +258,7 @@ def task(*args, **kwargs):
     # @task -- no options were (probably) given.
     # Also handles ctask's use case when given as @ctask, equivalent to
     # @task(obj, contextualized=True).
-    if len(args) == 1 and callable(args[0]):
+    if len(args) == 1 and callable(args[0]) and not isinstance(args[0], Task):
         return Task(args[0], **kwargs)
     # @task(pre, tasks, here)
     if args:
@@ -238,6 +276,7 @@ def task(*args, **kwargs):
     auto_shortflags = kwargs.pop('auto_shortflags', True)
     help = kwargs.pop('help', {})
     pre = kwargs.pop('pre', [])
+    post = kwargs.pop('post', [])
     # Handle unknown kwargs
     if kwargs:
         kwarg = (" unknown kwargs %r" % (kwargs,)) if kwargs else ""
@@ -253,7 +292,8 @@ def task(*args, **kwargs):
             default=default,
             auto_shortflags=auto_shortflags,
             help=help,
-            pre=pre
+            pre=pre,
+            post=post,
         )
         return obj
     return inner
@@ -267,3 +307,26 @@ def ctask(*args, **kwargs):
     """
     kwargs.setdefault('contextualized', True)
     return task(*args, **kwargs)
+
+
+class Call(object):
+    """
+    Represents a call/execution of a `.Task` with some arguments.
+
+    Wraps its `.Task` so it can be treated as one by `.Executor`.
+
+    Similar to `~functools.partial` with some added functionality (such as the
+    delegation to the inner task, and optional tracking of a given name.)
+    """
+    def __init__(self, task, *args, **kwargs):
+        self.task = task
+        self.args = args
+        self.kwargs = kwargs
+        self.name = None # Mostly manipulated by client code
+
+    def __getattr__(self, name):
+        return getattr(self.task, name)
+
+
+# Convenience/aesthetically pleasing-ish alias
+call = Call
