@@ -1,10 +1,10 @@
 import os
 import sys
 
-from spec import eq_, skip, Spec, ok_, trap
-from mock import patch
+from spec import eq_, skip, Spec, ok_, trap, raises
+from mock import patch, Mock
 
-from invoke.cli import parse, dispatch
+from invoke.cli import parse
 from invoke.context import Context
 from invoke.runner import run
 from invoke.parser import Parser
@@ -13,57 +13,39 @@ from invoke.tasks import task
 from invoke.exceptions import Failure
 import invoke
 
-from _utils import support
+from _utils import _dispatch, _output_eq, IntegrationSpec
 
 
-def _output_eq(cmd, expected):
-    return eq_(run(cmd).stdout, expected)
-
-
-class CLI(Spec):
+class CLI(IntegrationSpec):
     "Command-line behavior"
-    def setup(self):
-        os.chdir(support)
+    class basic_invocation:
+        @trap
+        def vanilla(self):
+            os.chdir('implicit')
+            _dispatch('inv foo')
+            eq_(sys.stdout.getvalue(), "Hm\n")
 
-    def _basic(self):
-        self.result = run("invoke -c integration print_foo", hide='both')
+        @trap
+        def vanilla_with_explicit_collection(self):
+            # Duplicates _output_eq above, but this way that can change w/o
+            # breaking our expectations.
+            _dispatch('inv -c integration print_foo')
+            eq_(sys.stdout.getvalue(), "foo\n")
 
-    # Yo dogfood, I heard you like invoking
-    @trap
-    def basic_invocation(self):
-        self._basic()
-        _output_eq("invoke -c integration print_foo", "foo\n")
+        def args(self):
+            _output_eq('-c integration print_name --name inigo', "inigo\n")
 
-    @trap
-    def implicit_task_module(self):
-        # Contains tasks.py
-        os.chdir('implicit')
-        # Doesn't specify --collection
-        _output_eq("invoke foo", "Hm\n")
-
-    @trap
-    def invocation_with_args(self):
-        _output_eq(
-            "invoke -c integration print_name --name whatevs",
-            "whatevs\n"
-        )
-
-    @trap
-    def invocation_with_underscored_args(self):
-        _output_eq(
-            "invoke -c integration print_underscored_arg --my-option whatevs",
-            "whatevs\n"
-        )
+        def underscored_args(self):
+            _output_eq(
+                '-c integration print_underscored_arg --my-option whatevs',
+                "whatevs\n",
+            )
 
     def contextualized_tasks_are_given_parser_context_arg(self):
         # go() in contextualized.py just returns its initial arg
-        retval = dispatch(['-c', 'contextualized', 'go'])[0]
+        retval = list(_dispatch('invoke -c contextualized go').values())[0]
         assert isinstance(retval, Context)
 
-    @trap
-    def shorthand_binary_name(self):
-        _output_eq("invoke -c integration print_foo", "foo\n")
-
     def core_help_option_prints_core_help(self):
         # TODO: change dynamically based on parser contents?
         # e.g. no core args == no [--core-opts],
@@ -80,6 +62,7 @@ Core options:
   --no-dedupe                      Disable task deduplication.
   -c STRING, --collection=STRING   Specify collection name to load. May be
                                    given >1 time.
+  -d, --debug                      Enable debug output.
   -e, --echo                       Echo executed commands before running.
   -h [STRING], --help[=STRING]     Show core or per-task help and exit.
   -H STRING, --hide=STRING         Set default value of run()'s 'hide' kwarg.
@@ -92,12 +75,9 @@ Core options:
                                    commands fail.
 
 """.lstrip()
-        r1 = run("inv -h", hide='out')
-        r2 = run("inv --help", hide='out')
-        eq_(r1.stdout, expected)
-        eq_(r2.stdout, expected)
+        for flag in ['-h', '--help']:
+            _output_eq(flag, expected)
 
-    @trap
     def per_task_help_prints_help_for_task_only(self):
         expected = """
 Usage: inv[oke] [--core-opts] punch [--options] [other tasks here ...]
@@ -110,12 +90,9 @@ Options:
   -w STRING, --who=STRING   Who to punch
 
 """.lstrip()
-        r1 = run("inv -c decorator -h punch", hide='out')
-        r2 = run("inv -c decorator --help punch", hide='out')
-        eq_(r1.stdout, expected)
-        eq_(r2.stdout, expected)
+        for flag in ['-h', '--help']:
+            _output_eq('-c decorator {0} punch'.format(flag), expected)
 
-    @trap
     def per_task_help_works_for_unparameterized_tasks(self):
         expected = """
 Usage: inv[oke] [--core-opts] biz [other tasks here ...]
@@ -127,33 +104,50 @@ Options:
   none
 
 """.lstrip()
-        r = run("inv -c decorator -h biz", hide='out')
-        eq_(r.stdout, expected)
+        _output_eq('-c decorator -h biz', expected)
 
-    @trap
     def per_task_help_displays_docstrings_if_given(self):
         expected = """
 Usage: inv[oke] [--core-opts] foo [other tasks here ...]
 
 Docstring:
   Foo the bar.
-  
+
 Options:
   none
 
 """.lstrip()
-        r = run("inv -c decorator -h foo", hide='out')
-        eq_(r.stdout, expected)
+        _output_eq('-c decorator -h foo', expected)
+
+    def per_task_help_dedents_correctly(self):
+        expected = """
+Usage: inv[oke] [--core-opts] foo2 [other tasks here ...]
+
+Docstring:
+  Foo the bar:
+
+    example code
+
+  Added in 1.0
+
+Options:
+  none
+
+""".lstrip()
+        _output_eq('-c decorator -h foo2', expected)
+
+    def version_info(self):
+        _output_eq('-V', "Invoke %s\n" % invoke.__version__)
 
     @trap
-    def version_info(self):
-        eq_(run("invoke -V").stdout, "Invoke %s\n" % invoke.__version__)
-
+    def version_override(self):
+        _dispatch('notinvoke -V', version="nope 1.0")
+        eq_(sys.stdout.getvalue(), "nope 1.0\n")
 
     class task_list:
         "--list"
 
-        def _listing(self, *lines):
+        def _listing(self, lines):
             return ("""
 Available tasks:
 
@@ -161,105 +155,113 @@ Available tasks:
 
 """ % '\n'.join("  " + x for x in lines)).lstrip()
 
-        @trap
+        def _list_eq(self, collection, listing):
+            cmd = '-c {0} --list'.format(collection)
+            _output_eq(cmd, self._listing(listing))
+
         def simple_output(self):
-            expected = self._listing(
+            expected = self._listing((
                 'bar',
+                'biz',
+                'boz',
                 'foo',
+                'post1',
+                'post2',
                 'print_foo',
                 'print_name',
                 'print_underscored_arg',
-            )
+            ))
             for flag in ('-l', '--list'):
-                eq_(run("invoke -c integration %s" % flag).stdout, expected)
+                _output_eq('-c integration {0}'.format(flag), expected)
 
-        @trap
         def namespacing(self):
-            # TODO: break out the listing behavior into a testable method, down
-            # with subprocesses!
-            expected = self._listing(
+            self._list_eq('namespacing', (
                 'toplevel',
                 'module.mytask',
-            )
-            eq_(run("invoke -c namespacing --list").stdout, expected)
+            ))
 
-        @trap
         def top_level_tasks_listed_first(self):
-            expected = self._listing(
+            self._list_eq('simple_ns_list', (
                 'z_toplevel',
                 'a.subtask'
-            )
-            eq_(run("invoke -c simple_ns_list --list").stdout, expected)
+            ))
 
-        @trap
         def subcollections_sorted_in_depth_order(self):
-            expected = self._listing(
+            self._list_eq('deeper_ns_list', (
                 'toplevel',
                 'a.subtask',
                 'a.nother.subtask',
-            )
-            eq_(run("invoke -c deeper_ns_list --list").stdout, expected)
+            ))
 
-        @trap
         def aliases_sorted_alphabetically(self):
-            expected = self._listing(
+            self._list_eq('alias_sorting', (
                 'toplevel (a, z)',
-            )
-            eq_(run("invoke -c alias_sorting --list").stdout, expected)
+            ))
 
-
-        @trap
         def default_tasks(self):
             # sub-ns default task display as "real.name (collection name)"
-            expected = self._listing(
+            self._list_eq('explicit_root', (
                 'top_level (othertop)',
                 'sub.sub_task (sub, sub.othersub)',
-            )
-            eq_(run("invoke -c explicit_root --list").stdout, expected)
+            ))
 
-    @trap
-    def no_deduping(self):
-        expected = """
-foo
-foo
-bar
-""".lstrip()
-        eq_(run("invoke -c integration --no-dedupe foo bar").stdout, expected)
+        def docstrings_shown_alongside(self):
+            self._list_eq('docstrings', (
+                'leading_whitespace    foo',
+                'no_docstring',
+                'one_line              foo',
+                'two_lines             foo',
+                'with_aliases (a, b)   foo',
+            ))
+
+    def debug_flag_activates_logging(self):
+        # Have to patch our logger to get in before Nose logcapture kicks in.
+        with patch('invoke.util.debug') as debug:
+            _dispatch('inv -d -c debugging foo')
+            debug.assert_called_with('my-sentinel')
 
     class run_options:
         "run() related CLI flags"
         def _test_flag(self, flag, kwarg, value):
             with patch('invoke.context.run') as run:
-                dispatch(flag + ['-c', 'contextualized', 'run'])
+                _dispatch('invoke {0} -c contextualized run'.format(flag))
                 run.assert_called_with('x', **{kwarg: value})
 
         def warn_only(self):
-            self._test_flag(['-w'], 'warn', True)
+            self._test_flag('-w', 'warn', True)
 
         def pty(self):
-            self._test_flag(['-p'], 'pty', True)
+            self._test_flag('-p', 'pty', True)
 
         def hide(self):
-            self._test_flag(['--hide', 'both'], 'hide', 'both')
+            self._test_flag('--hide both', 'hide', 'both')
 
         def echo(self):
-            self._test_flag(['-e'], 'echo', True)
+            self._test_flag('-e', 'echo', True)
 
 
 TB_SENTINEL = 'Traceback (most recent call last)'
 
 class HighLevelFailures(Spec):
-
+    @trap
     def command_failure(self):
         "Command failure doesn't show tracebacks"
-        result = run("inv -c fail simple", warn=True, hide='both')
-        assert TB_SENTINEL not in result.stderr
-        assert result.exited != 0
+        with patch('sys.exit') as exit:
+            _dispatch('inv -c fail simple')
+            assert TB_SENTINEL not in sys.stderr.getvalue()
+            exit.assert_called_with(1)
 
     class parsing:
+        @trap
         def should_not_show_tracebacks(self):
-            result = run("inv -c fail missing_pos", warn=True, hide='both')
-            assert TB_SENTINEL not in result.stderr
+            # Ensure we fall out of dispatch() on missing parser args,
+            # but are still able to look at stderr to ensure no TB got printed
+            with patch('sys.exit', Mock(side_effect=SystemExit)):
+                try:
+                    _dispatch("inv -c fail missing_pos")
+                except SystemExit:
+                    pass
+                assert TB_SENTINEL not in sys.stderr.getvalue()
 
         def should_show_core_usage_on_core_failures(self):
             skip()
@@ -398,7 +400,7 @@ class CLIParsing(Spec):
     def globbed_shortflags_with_multipass_parsing(self):
         "mytask -cb and -bc"
         for args in ('-bc', '-cb'):
-            _, _, r = parse(['mytask4', args], self.c)
+            _, _, r = parse(['invoke', 'mytask4', args], self.c)
             a = r[0].args
             eq_(a.clean.value, True)
             eq_(a.browse.value, True)
